import React, { forwardRef, useEffect, useImperativeHandle, useMemo, } from 'react';
import { shallow } from 'zustand/shallow';
import { StoreContext, createGeschichte, } from '../../store.js';
import { createSearch } from '../../utils.js';
const handleHistoryV4 = (update, action) => {
    return {
        action,
        location: update,
    };
};
const handleHistoryV5 = ({ location, action, }) => {
    return {
        action,
        location,
    };
};
let handler;
export const handleHistoryEvent = (action) => {
    if (handler) {
        return handler;
    }
    if (action) {
        return (handler = handleHistoryV4);
    }
    return (handler = handleHistoryV5);
};
export const GeschichteWithHistory = forwardRef(({ children, history }, ref) => {
    const historyInstance = useMemo(() => {
        return {
            initialSearch: () => history.location.search,
            push: async (query) => {
                history.push({
                    hash: history.location.hash,
                    search: createSearch(query),
                }, { __g__: true });
            },
            replace: async (query) => history.replace({
                hash: history.location.hash,
                search: createSearch(query),
            }, { __g__: true }),
        };
    }, [history]);
    const value = useMemo(() => createGeschichte(historyInstance), [historyInstance]);
    const state = value(({ unregister, updateFromQuery, }) => ({
        unregister,
        updateFromQuery,
    }), shallow);
    useEffect(() => {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-ignore
        return history.listen((update, maybeAction) => {
            const { location, action } = handleHistoryEvent(maybeAction)(update, maybeAction);
            // don't handle our own actions
            if ((action === 'REPLACE' || action === 'PUSH') &&
                location.state &&
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                // @ts-ignore
                location.state.__g__) {
                return;
            }
            state.updateFromQuery(location.search);
        });
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, [history, state.updateFromQuery]);
    useImperativeHandle(ref, () => ({
        updateFromQuery: state.updateFromQuery,
    }), [state]);
    useEffect(() => {
        const { unregister } = state;
        return () => {
            return unregister();
        };
    }, [state]);
    return (React.createElement(StoreContext.Provider, { value: value }, children));
});
export default GeschichteWithHistory;
