'use client';
import { useSearchParams, useRouter, usePathname } from 'next/navigation.js';
import { StoreContext, createGeschichte, } from '../../store.js';
import React, { memo, useEffect, useMemo, useRef } from 'react';
import { shallow } from 'zustand/shallow';
import { createSearch } from '../../utils.js';
const GeschichteForNextAppRouter = ({ children }) => {
    const searchParams = useSearchParams();
    const { push, replace } = useRouter();
    const pathname = usePathname();
    const router = useRef({ push, replace, searchParams, pathname });
    const historyInstance = useMemo(() => {
        const { searchParams, push, replace, pathname } = router.current;
        return {
            initialSearch: () => searchParams,
            push: async (query) => {
                push(`${pathname}${createSearch(query)}`);
            },
            replace: async (query) => {
                replace(`${pathname}${createSearch(query)}`);
            },
        };
    }, []);
    const useStore = useMemo(() => createGeschichte(historyInstance), [historyInstance]);
    const state = useStore(({ unregister, updateFromQuery }) => ({
        unregister,
        updateFromQuery,
    }), shallow);
    useEffect(() => {
        if (router.current.searchParams !== searchParams) {
            state.updateFromQuery(searchParams);
        }
        router.current = { push, replace, searchParams, pathname };
        return () => {
            state.unregister();
        };
    }, [push, replace, pathname, searchParams, state]);
    return (React.createElement(StoreContext.Provider, { value: useStore }, children));
};
export default memo(GeschichteForNextAppRouter);
